// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions:	   
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            	:| Mod. Date 	:| Changes Made:
//   V1.0 :| Jeff Chou       	:| 				:| Initial Revision
// --------------------------------------------------------------------
//=============================================================================
// FIFO Declare
//=============================================================================
module ReadFIFO (
	aclr,
	data,
	rdclk,
	rdreq,
	wrclk,
	wrreq,
	q,
	rdempty,
	wrfull,
	wrusedw);
	parameter DATA_BITS = 32;
	parameter RDDW_NUM = 9;
	
	input	  aclr;
	input	[255:0]  data;
	input	  rdclk;
	input	  rdreq;
	input	  wrclk;
	input	  wrreq;
	output	[DATA_BITS-1:0]  q;
	output	  rdempty;
	output	  wrfull;
	output	[6:0]  wrusedw;

	wire  sub_wire0;
	wire [6:0] sub_wire1;
	wire  sub_wire2;
	wire [DATA_BITS-1:0] sub_wire3;
	wire  rdempty = sub_wire0;
	wire [6:0] wrusedw = sub_wire1[6:0];
	wire  wrfull = sub_wire2;
	wire [DATA_BITS-1:0] q = sub_wire3[DATA_BITS-1:0];

	dcfifo_mixed_widths	dcfifo_mixed_widths_component (
				.wrclk (wrclk),
				.rdreq (rdreq),
				.aclr (aclr),
				.rdclk (rdclk),
				.wrreq (wrreq),
				.data (data),
				.rdempty (sub_wire0),
				.wrusedw (sub_wire1),
				.wrfull (sub_wire2),
				.q (sub_wire3)
				// synopsys translate_off
				,
				.rdfull (),
				.rdusedw (),
				.wrempty ()
				// synopsys translate_on
				);
	defparam
		dcfifo_mixed_widths_component.intended_device_family = "Cyclone III",
		dcfifo_mixed_widths_component.lpm_hint = "MAXIMIZE_SPEED=7,",
		dcfifo_mixed_widths_component.lpm_numwords = 128,//64,
		dcfifo_mixed_widths_component.lpm_showahead = "ON",
		dcfifo_mixed_widths_component.lpm_type = "dcfifo",
		dcfifo_mixed_widths_component.lpm_width = 256,
		dcfifo_mixed_widths_component.lpm_widthu = 7,
		dcfifo_mixed_widths_component.lpm_widthu_r = RDDW_NUM + 1,
		dcfifo_mixed_widths_component.lpm_width_r = DATA_BITS,
		dcfifo_mixed_widths_component.overflow_checking = "ON",
		dcfifo_mixed_widths_component.rdsync_delaypipe = 5,
		dcfifo_mixed_widths_component.underflow_checking = "ON",
		dcfifo_mixed_widths_component.use_eab = "ON",
		dcfifo_mixed_widths_component.write_aclr_synch = "OFF",
		dcfifo_mixed_widths_component.wrsync_delaypipe = 5;


endmodule
//=======================================================================================















//=======================================================================================
//MAIN MODULE
//=======================================================================================
module DDR2_SODIMM_Read_Port (

				//////////// 	Clock and Reset_n		////////////////
				iCLK,
				iRST_n,
				
				////////////	Avalone Bus Signal  	////////////////
				oADDRESS,
				oCS,
				oREAD,
				iREAD_DATA,
				iREAD_VALID,
				oBURST_COUNT,
				iWAIT_REQ,
				
				/////////// 	User Fifo Interface		////////////////
				iRST_n_F,								// Enable this fifo interface
				iCLK_F,									// Reference Clock of FIFO Read Port 
				iREAD_ACK_F,								
				oREAD_DATA_F,
				oEMPTY_F,
				oPORT_READY_F,
				iIP_INIT_DONE
				
				
				,c_state,error
				//test
				//,n_state,c_state,data_count,wrusedw
					);

//test
//output	[3:0]wrusedw;
output	[3:0]c_state;	// current state
output	error;
//output	[2:0]n_state;	// next state
//output 	[3:0]data_count;

//

//==============================================================================
// PARAMETER declarations
//==============================================================================
parameter DATA_WIDTH_BITS = 64; 
parameter STARTING_ADDRESS = 0;
parameter PORT_SIZE_BYTES = 1104000;
//localparam END_ADDR = STARTING_ADDRESS + PORT_SIZE_BYTES - BURST_COUNT*32;
parameter BURST_COUNT = 8;					// 4 	
localparam DATA_CNT = BURST_COUNT;
localparam INC_ADDR = BURST_COUNT* 32;			

localparam rddw_num = 	(DATA_WIDTH_BITS == 8) 	?		11	:
						(DATA_WIDTH_BITS == 16) ?		10	:	
						(DATA_WIDTH_BITS == 32) ?		9	:
						(DATA_WIDTH_BITS == 64) ?		8	:
						(DATA_WIDTH_BITS == 128)?		7	:
						(DATA_WIDTH_BITS == 256)?		6	:	11;

//==============================================================================
// PORT declarations
//==============================================================================
input	iCLK;
input	iRST_n;

output	[31:0]oADDRESS;
output	oCS;
output	oREAD;
input	[255:0]iREAD_DATA;
input	iREAD_VALID;
output	[7:0]oBURST_COUNT;
input	iWAIT_REQ;

input 	iCLK_F;
input	iRST_n_F;
input	iREAD_ACK_F;
output	[DATA_WIDTH_BITS-1 : 0]oREAD_DATA_F;
output 	oEMPTY_F;
input	iIP_INIT_DONE;
output	oPORT_READY_F;

//=============================================================================
// REG/WIRE declarations
//=============================================================================
//reg		[31:0]oADDRESS;
//reg		oCS;
//reg		oREAD;
wire	rstn;


//=============================================================================
// Structural coding
//=============================================================================

wire	[31:0]END_ADDR;
assign	END_ADDR = STARTING_ADDRESS + PORT_SIZE_BYTES - BURST_COUNT*32;

wire	[31:0]SINGLE_END_ADDR;
assign	SINGLE_END_ADDR = STARTING_ADDRESS + PORT_SIZE_BYTES - 32;

wire	[31:0]REAL_END_ADDRESS;
assign	REAL_END_ADDRESS = STARTING_ADDRESS + PORT_SIZE_BYTES;

wire	[31:0]next_address;
assign	next_address = oADDRESS + INC_ADDR;




	// Ready to be read
	reg	oPORT_READY_F;
	always@(posedge iCLK)
		begin
			if (~rstn)
				oPORT_READY_F <= 0;
			else if (~iRST_n_F)
				oPORT_READY_F <= 0;
			else if (wrusedw == BURST_COUNT)
				oPORT_READY_F <= 1;
		end


	// Avalone Bus Burst count
//	assign 	oBURST_COUNT =	(~last_rd_flag)? 8'h20 : end_rm_bytes[12:5];
	
							
							
	assign	rstn = ( (iRST_n) & (iIP_INIT_DONE) & (iRST_n_F) );
	//	FIFO Declaration
	wire [6:0]wrusedw;
	ReadFIFO rFIFO(
			
			.aclr(~iRST_n_F),	
			.data(iREAD_DATA),
			.rdclk(iCLK_F),
			.rdreq(iREAD_ACK_F),
			.wrclk(iCLK),
			.wrreq(iREAD_VALID),
			.q(oREAD_DATA_F),
			.rdempty(oEMPTY_F),
			.wrfull(),
			.wrusedw(wrusedw)
			
				);
	defparam
		rFIFO.DATA_BITS = DATA_WIDTH_BITS,
		rFIFO.RDDW_NUM = rddw_num;


	//	Address add one stage enable
	reg	add_one_flag;
//	always@(posedge iCLK)
//		begin
//			if (~iRST_n)
//				add_one_flag <= 0;
//			else if (n_state == 5)
//				add_one_flag <= 1;
//			else if (c_state == 6)
//				add_one_flag <= 0;
//		end

	//	last read remain bytes
	wire	[31:0]end_rm_bytes;
	assign	end_rm_bytes = REAL_END_ADDRESS - nor_address;
	
	wire	[12:0]add_rm_address;
	assign	add_rm_address = {end_rm_bytes[12:5], 5'h0};
	
	reg		[7:0]last_burst_cnt;
	always@(posedge iCLK)
		begin
			if (c_state == 5)
				last_burst_cnt <= end_rm_bytes[12:5];
		end
	
	//	last read flag
	reg	[7:0]oBURST_COUNT;
	reg	last_rd_flag;
	always@(posedge iCLK)
		begin
			if (~iRST_n)
				oBURST_COUNT <= BURST_COUNT;
			else if (c_state == 5)
				oBURST_COUNT <= end_rm_bytes[12:5];
			else if (c_state == 8)
				oBURST_COUNT <= BURST_COUNT;
		end

	

	//	update state (memory read operation)
	reg	[3:0]c_state;	// current state
	reg	[3:0]n_state;	// next state

	always@(posedge iCLK)
		begin
			if ( (~rstn) || (~iRST_n_F) )
				c_state <= 0;
			else
				c_state <= n_state;
		end
	
	//	FSM description	
	reg	[7:0]data_count;
	always@(c_state or wrusedw or data_count or iWAIT_REQ)	
		begin
			case (c_state)
				0:	n_state <= 1;
				1:	
					if (wrusedw < DATA_CNT)
						begin
							if (next_address > REAL_END_ADDRESS)
								n_state <= 5;
							else
								n_state <= 2;
						end
					else
						n_state <= 1;	
				2:
					if (~iWAIT_REQ)
						n_state <= 3;
					else
						n_state <= 2;
				3:
					if (data_count == BURST_COUNT)
						n_state <= 4;
					else
						n_state <= 3;
				4:	n_state <= 1;	
				5:	n_state <= 6;	
				6:	
					if (~iWAIT_REQ)
						n_state <= 7;
					else
						n_state <= 6;
				7:
					if (data_count == last_burst_cnt)
						n_state <= 8;
					else
						n_state <= 7;
				8:	n_state <= 1;	
						
				
//				5:	
//					if (add1_counter == BURST_COUNT)
//						n_state <= 6;
//					else
//						n_state <= 5;
//				6:	n_state <= 7;
//				7:	
//					if (data_count == BURST_COUNT)
//						n_state <= 1;
//					else
//						n_state <= 7;
				default:n_state <= 0;
			endcase
		end

	//	Read signal description
	reg	lut_read;
	always@(c_state)
		begin
			case (c_state)
				0:	lut_read <= 0;
				1:	lut_read <= 0;
				2:	lut_read <= 1;
				3:	lut_read <= 0;
				4:	lut_read <= 0;
				5:	lut_read <= 0;
				6:	lut_read <= 1;
				7:	lut_read <= 0;
				8:	lut_read <= 0;
//				5:	
//					if (add1_counter == BURST_COUNT)
//						lut_read <= 0;
//					else
//						lut_read <= 1;
//				6:	lut_read <= 0;
				default	lut_read <= 0;
			endcase
		end
	assign	oREAD = lut_read;
	assign	oCS = lut_read;

	// add-one data counter
	reg	[31:0]add1_counter;
	always@(posedge iCLK)
		begin
			if (~rstn) 
				add1_counter <= 6'b00000;
			else if (c_state == 1)
				add1_counter <= 6'b00000;	
			else if (add1_counter == BURST_COUNT)
				add1_counter <= 6'b00000;
			else if ( (lut_read) & (~iWAIT_REQ) )
				add1_counter <= add1_counter + 6'b00001;
		end

	//	add-one address
	reg	[31:0]add1_address;
	always@(posedge iCLK)
		begin
			if ( (~rstn) || (~iRST_n_F) )
				add1_address <= STARTING_ADDRESS;
			else if (c_state == 0)
				add1_address <= STARTING_ADDRESS;	
			else if ( (lut_read) & (~iWAIT_REQ) )
				begin
//					if (add1_address == SINGLE_END_ADDR)//if (oADDRESS == END_ADDR)
//						add1_address <= STARTING_ADDRESS;	// 2008-11-04 add
					if (c_state == 2) 
						begin
							if (add1_address == END_ADDR)
								add1_address <= STARTING_ADDRESS;
							else
								add1_address <= add1_address + INC_ADDR;
						end
					else 
						begin
							if (add1_address == SINGLE_END_ADDR)
								add1_address <= STARTING_ADDRESS;
							else
								add1_address <= add1_address + 32;
						end
				end
			
		end


	assign	oADDRESS = nor_address;
	
	wire	[31:0]end_rm_address;
	assign	end_rm_address = REAL_END_ADDRESS - add_rm_address;
	
	//	Address signal description
	reg	[31:0]nor_address;
	always@(posedge iCLK)
		begin
			if ( (~rstn) || (~iRST_n_F) )
				nor_address <= STARTING_ADDRESS;
			else if (c_state == 0)
				nor_address <= STARTING_ADDRESS;
//			else if (c_state == 6)
//				nor_address <= add1_address;
			else if (c_state == 4)
				begin
					if (nor_address == END_ADDR)//if (oADDRESS == END_ADDR)
						nor_address <= STARTING_ADDRESS;
					else
						nor_address <= nor_address + INC_ADDR;
				end
			else if (c_state == 8)
				begin
					if (nor_address == end_rm_address)
						nor_address <= STARTING_ADDRESS;
					else
						nor_address <= nor_address + add_rm_address;
				end
		end


	// data read counter
	always@(posedge iCLK)
		begin
			if (~rstn) 
				data_count <= 6'b00000;
			else if (c_state == 1)
				data_count <= 6'b00000;	
//			else if (data_count == BURST_COUNT)
//				data_count <= 6'b00000;
			else if (iREAD_VALID)
				data_count <= data_count + 6'b00001;
		end






endmodule 
