// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions:	   
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            	:| Mod. Date 	:| Changes Made:
//   V1.0 :| Jeff Chou       	:| 				:| Initial Revision
// --------------------------------------------------------------------
//=============================================================================
// FIFO Declare
//=============================================================================
// 256 bits(SO-DIMM Data Depth) * 16(FIFO Deep) = DATA_BITS * lpm_numwords,
// 
module WriteFIFO (
	aclr,
	data,
	rdclk,
	rdreq,
	wrclk,
	wrreq,
	q,
	rdempty,
	rdusedw,
	wrfull,
	wrusedw);
	
	parameter	DATA_WIDTH_BITS = 32;
//	parameter	RDDW_NUM	=	5;
//	parameter	FIFO_DEPTH  	= 64;
//	localparam 	DEPTH_NUM		= 	(FIFO_DEPTH == 64) ?		6	:
//									(FIFO_DEPTH == 128)?		7	:
//									(FIFO_DEPTH == 256)?		8	:	
//									(FIFO_DEPTH == 512)?		9	:	9;
	parameter 	LOCAL_FIFO_DEPTH = 1024;
	parameter	LOCAL_NUM = 10;								
			
															
											
	input	  aclr;
	input	[DATA_WIDTH_BITS-1:0]  data;
	input	  rdclk;
	input	  rdreq;
	input	  wrclk;
	input	  wrreq;
	output	[255:0]  q;
	output	  rdempty;
	output	[6:0]  rdusedw;
	output	  wrfull;
	output	[10:0]  wrusedw;

	wire  sub_wire0;
	wire [10:0] sub_wire1;
	wire  sub_wire2;
	wire [255:0] sub_wire3;
	wire [6:0] sub_wire4;
	wire  rdempty = sub_wire0;
	wire [10:0] wrusedw = sub_wire1[10:0];
	wire  wrfull = sub_wire2;
	wire [255:0] q = sub_wire3[255:0];
	wire [6:0] rdusedw = sub_wire4[6:0];

	dcfifo_mixed_widths	dcfifo_mixed_widths_component (
				.wrclk (wrclk),
				.rdreq (rdreq),
				.aclr (aclr),
				.rdclk (rdclk),
				.wrreq (wrreq),
				.data (data),
				.rdempty (sub_wire0),
				.wrusedw (sub_wire1),
				.wrfull (sub_wire2),
				.q (sub_wire3),
				.rdusedw (sub_wire4)
				// synopsys translate_off
				,
				.rdfull (),
				.wrempty ()
				// synopsys translate_on
				);
	defparam
		dcfifo_mixed_widths_component.intended_device_family = "Cyclone III",
		dcfifo_mixed_widths_component.lpm_hint = "MAXIMIZE_SPEED=7,",
		dcfifo_mixed_widths_component.lpm_numwords = LOCAL_FIFO_DEPTH,
		dcfifo_mixed_widths_component.lpm_showahead = "ON",
		dcfifo_mixed_widths_component.lpm_type = "dcfifo",
		dcfifo_mixed_widths_component.lpm_width = DATA_WIDTH_BITS,
		dcfifo_mixed_widths_component.lpm_widthu = LOCAL_NUM,
		dcfifo_mixed_widths_component.lpm_widthu_r = 7,
		dcfifo_mixed_widths_component.lpm_width_r = 256,
		dcfifo_mixed_widths_component.overflow_checking = "ON",
		dcfifo_mixed_widths_component.rdsync_delaypipe = 5,
		dcfifo_mixed_widths_component.underflow_checking = "ON",
		dcfifo_mixed_widths_component.use_eab = "ON",
		dcfifo_mixed_widths_component.write_aclr_synch = "OFF",
		dcfifo_mixed_widths_component.wrsync_delaypipe = 5;


endmodule












//=======================================================================================
//MAIN MODULE
//=======================================================================================
module DDR2_SODIMM_Write_Port (

				//////////// 	Clock and Reset_n		////////////////
				iCLK,
				iRST_n,
				
				////////////	Avalon Bus Signal  	////////////////
				oADDRESS,
				oCS,
				oWRITE,
				oWRITE_DATA,
				oBYTE_ENABLE,
				iWAIT_REQ,
				
				/////////// 	User Fifo Interface		////////////////
				iRST_n_F,								// Enable this fifo interface
				iCLK_F,									// Reference Clock of FIFO Read Port 
				iWRITE_F,								
				iWRITE_DATA_F,
				oFULL_F,
				
				iFLUSH_REQ_F,								// Flush FIFO request
				
				oFLUSH_BUSY_F,								// Flush busy signal
				
				iIP_INIT_DONE,
				
				
				
				oBURST_COUNT
				
				
				//,write
				,c_state_flush,wrusedw
				,c_state
				//test
//				,data_cnt,c_state,n_state,rdusedw,fifo_read
				//,test_fifo_read,c_state
					);
output	[4:0]c_state_flush;
output	[8:0]wrusedw;
output	[3:0]c_state;
//test
//output	[3:0]data_cnt;
//output	[3:0]c_state;
//output	[3:0]n_state;
//output	[3:0]rdusedw;
//output	fifo_read;
//

//==============================================================================
// PARAMETER declarations
//==============================================================================
parameter DATA_WIDTH_BITS = 64; 
parameter STARTING_ADDRESS = 0;
parameter PORT_SIZE_BYTES = 1104000;
parameter BURST_CNT = 8;


localparam 	DDR_NUM  = 128;
localparam 	rddw_num = 6; // depend on DDR_NUM
//localparam 	FIFO_DEPTH = 512;//4096

//localparam	FLAG_1	= 	(FIFO_DEPTH == 512)? 0 :
//						(FIFO_DEPTH == 256)? 1 :
//						(FIFO_DEPTH == 128)? 2 :
//						(FIFO_DEPTH == 64)? 3 : 3;


localparam END_ADDR = STARTING_ADDRESS + PORT_SIZE_BYTES - BURST_CNT * 32;
localparam BYTE_EN_PARA = DATA_WIDTH_BITS / 8;
localparam	half_of_depth = BURST_CNT;//32;//((DATA_WIDTH_BITS*FIFO_DEPTH)/256)/2 ;

localparam INC_ADDR = BURST_CNT * 32;
localparam MEM_LOCAL_WIDTH_BITS = 256;
localparam MEM_LOCAL_WIDTH_BYTES = MEM_LOCAL_WIDTH_BITS / 8;

localparam FLUSH_END_ADDRESS = STARTING_ADDRESS + PORT_SIZE_BYTES - 32;
localparam REAL_END_ADDRESS = STARTING_ADDRESS + PORT_SIZE_BYTES;

//localparam rddw_num = 	(DATA_WIDTH_BITS == 8) 	?		3 - FLAG_1	:
//						(DATA_WIDTH_BITS == 16) ?		4 - FLAG_1	:	
//						(DATA_WIDTH_BITS == 32) ?		5 - FLAG_1	:
//						(DATA_WIDTH_BITS == 64) ?		6 - FLAG_1	:
//						(DATA_WIDTH_BITS == 128)?		7 - FLAG_1	:
//						(DATA_WIDTH_BITS == 256)?		8 - FLAG_1	:	8;

localparam LOCAL_FIFO_DEPTH = 	(DATA_WIDTH_BITS == 8) 	?		4096	:
								(DATA_WIDTH_BITS == 16) ?		2048	:	
								(DATA_WIDTH_BITS == 32) ?		1024	:
								(DATA_WIDTH_BITS == 64) ?		512		:
								(DATA_WIDTH_BITS == 128)?		256		:
								(DATA_WIDTH_BITS == 256)?		128		:	1024;
								
localparam LOCAL_NUM = 	(DATA_WIDTH_BITS == 8) 	?		12	:
						(DATA_WIDTH_BITS == 16) ?		11	:	
						(DATA_WIDTH_BITS == 32) ?		10	:
						(DATA_WIDTH_BITS == 64) ?		9	:
						(DATA_WIDTH_BITS == 128)?		8	:
						(DATA_WIDTH_BITS == 256)?		7	:	10;								

//==============================================================================
// PORT declarations
//==============================================================================
input	iCLK;
input	iRST_n;

output	[31:0]oADDRESS;
output	oCS;
output	oWRITE;
output	[255:0]oWRITE_DATA;
input	iWAIT_REQ;
output	[31:0]oBYTE_ENABLE;

input 	iCLK_F;
input	iRST_n_F;
input	iWRITE_F;
input	[DATA_WIDTH_BITS-1 : 0]iWRITE_DATA_F;
output 	oFULL_F;

input	iFLUSH_REQ_F;	// Flush FIFO request
output	oFLUSH_BUSY_F;	// Flush busy signal

input	iIP_INIT_DONE;	
output	[7:0]oBURST_COUNT;



//test
//output	[3:0]c_state;			
//=============================================================================
// REG/WIRE declarations
//=============================================================================
//reg		[31:0]oADDRESS;	
//reg		[31:0]oBYTE_ENABLE;				
wire	rstn;

//=============================================================================
// Structural coding
//=============================================================================

	wire	[8:0]wrusedw;
	
	wire	fifo_read;
	wire	[rddw_num:0]rdusedw;
	assign	fifo_read = ( oWRITE & (~iWAIT_REQ) );
	assign	rstn = iRST_n & iIP_INIT_DONE;
	
	wire	fifo_write;
	assign	fifo_write = ( oFLUSH_BUSY_F ) ?  0 : iWRITE_F;
	
	reg	[rddw_num:0]flush_cnt;
	always@(posedge iCLK)
		begin
			if ( (~iRST_n) || (~iRST_n_F) )
				flush_cnt <= 0;
			else if ( (c_state_flush == 2) || (c_state_flush == 3) )
				begin
					if (fifo_read)
						flush_cnt <= flush_cnt + 1;
					else
						flush_cnt <= flush_cnt;
				end
			else if (c_state_flush == 8)
				flush_cnt <= 0;
			
		end
	
	reg	[rddw_num:0]usdw_cnt;
	always@(posedge iCLK)
		begin
			if (c_state_flush == 2)
				usdw_cnt <= rdusedw;
		end
	
	
	WriteFIFO wFIFO(
		.aclr(~iRST_n_F),
		.data(iWRITE_DATA_F),
		.rdclk(iCLK),
		.rdreq( fifo_read ),
		.wrclk(iCLK_F),
		.wrreq(fifo_write),
		.q(oWRITE_DATA),
		.rdempty(),
		.rdusedw(rdusedw),
		.wrfull(oFULL_F),
		.wrusedw(wrusedw)
			);
	defparam
		wFIFO.DATA_WIDTH_BITS = DATA_WIDTH_BITS,
		wFIFO.LOCAL_NUM		  = LOCAL_NUM,
		wFIFO.LOCAL_FIFO_DEPTH	  = LOCAL_FIFO_DEPTH;
			
		

//	parameter	DATA_WIDTH_BITS = 32;
//	parameter	RDDW_NUM	=	5;


	assign	oWRITE 	=	((oFLUSH_BUSY_F) & (c_state == 1))?  	oWRITE_fls 	:	WRITE_normal;
	assign	oCS		=	((oFLUSH_BUSY_F) & (c_state == 1))?	oCS_fls		:	CS_normal	;
	assign	oADDRESS =  norm_address;//((oFLUSH_BUSY_F) & (c_state == 1))?	flush_address:	
						//(crs_addr_flag)? 	flush_address: 	norm_address;
						
	assign	oBURST_COUNT =( (oFLUSH_BUSY_F) & (c_state == 1) )? flush_burst_cnt	:
						  (crs_addr_flag)?  						1			:	
						  (rm_burst_flag)?					end_rm_bytes[12:5]	:	
						  (last_word)?								1			: BURST_CNT;


	/////////////////////////////////////////////
	// Description of FIFO flush control circuit 
	reg	delay1_flush;
	reg	delay2_flush;
	reg	delay3_flush;
	reg	delay4_flush;
	always@(posedge iCLK_F)
		begin
			delay1_flush <= iFLUSH_REQ_F;
			delay2_flush <= delay1_flush;
			delay3_flush <= delay2_flush;
			delay4_flush <= delay3_flush;
		end
	
	reg		flush_busy;
	always@(posedge iCLK_F)
		begin
			if ( (~iRST_n) || ( flush_done ) )
				begin
					flush_busy <= 0;
				end
			else if ( delay4_flush )
				begin
					flush_busy <= 1;
				end
		end	
	
	// First Response 
	reg		oFLUSH_BUSY_F;
	always@(posedge iCLK_F)
		begin
			if ( (~iRST_n) || ( flush_done ) )
				begin
					oFLUSH_BUSY_F <= 0;
				end
			else if ( iFLUSH_REQ_F )
				begin
					oFLUSH_BUSY_F <= 1;
				end
		end
	
	reg			[4:0]c_state_flush;
	reg			[4:0]n_state_flush;
	always@(posedge iCLK)
		begin
			if ( (~iRST_n) || (~iRST_n_F) )
				c_state_flush <= 0;
			else
				c_state_flush <= n_state_flush;
		end

	always@(c_state_flush or flush_busy or rdusedw or usedw_cnt)
		begin
			case (c_state_flush)
				0:	n_state_flush <= 1;
				1:
					begin
						if ( (flush_busy) & (rdusedw == 0) ) 
							n_state_flush <= 2;
						else if ( (c_state == 1) & (rdusedw < half_of_depth) )
							begin
								if ( (flush_busy) & (rdusedw !== 0) )
									n_state_flush <= 9;
								else
									n_state_flush <= 1;
							end
						else
							n_state_flush <= 1;
					end
				2:	n_state_flush <= 3;
				3:	n_state_flush <= 4;	
				4:
					begin
						if (usedw_cnt == 15)
							n_state_flush <= 5;
						else
							n_state_flush <= 4;
					end
				5:
					begin
						if (wrusedw == 0)
							n_state_flush <= 8;
						else 
							n_state_flush <= 6;
					end
				6:	n_state_flush <= 7;
				7:	
					begin
						if (~iWAIT_REQ) 
							n_state_flush <= 8;
						else
							n_state_flush <= 7;
					end
				8:	
					begin
						if (~flush_busy)
							n_state_flush <= 1;
						else
							n_state_flush <= 8;
					end
					
				9:
					begin
						if (nor_rm_addr <= REAL_END_ADDRESS)
							n_state_flush <= 10;
						else
							n_state_flush <= 14;
					end
				10:	n_state_flush <= 11;		// burst count <= remain bytes of FIFO (rdusedw)
				11:	n_state_flush <= 12;		// Write <= 1;
				12:
					begin
//						if (~iWAIT_REQ) 
//							begin
								if (data_cnt == flush_burst_cnt)
									n_state_flush <= 13;
								else
									n_state_flush <= 12;
//							end
//						else
//							n_state_flush <= 12;
					end
				13:
					begin
						//n_state_flush <= 5;		// to detect the wrusedw == 0 ??
						n_state_flush <= 19;
					end
				14:	n_state_flush <= 15;		//	burst count <= real_end_addr - normal_addr
				15:	n_state_flush <= 16;		// Write <= 1;
				16:
					begin
//						if (~iWAIT_REQ) 
//							begin
								if (data_cnt == flush_burst_cnt)
									n_state_flush <= 17;
								else
									n_state_flush <= 16;
//							end
//						else
//							n_state_flush <= 16;
					end
				17:	n_state_flush <= 18;
				18: n_state_flush <= 9;
				19: n_state_flush <= 5;
					
//				4:	
//					begin
//						if (~flush_busy)
//							n_state_flush <= 1;
//						else
//							n_state_flush <= 4;
//					end
				default:	n_state_flush <= 0;
			endcase
		end
		
	//////////////////////////////////////////////////////	
		
	reg	write_fls;
	always@(c_state_flush or rdusedw or iWAIT_REQ)
		begin
			case (c_state_flush)
				0:	write_fls <= 0;
				1:	write_fls <= 0;
				2:	write_fls <= 1;
				3:	
					begin
						if ( flush_cnt == usdw_cnt )
							write_fls <= 0;
						else
							write_fls <= 1;
					end
				4:	write_fls <= 0;
				5:	write_fls <= 0;
				6:	write_fls <= 1;
				7:	
					begin
						if (~iWAIT_REQ) 
							write_fls <= 0;
						else
							write_fls <= 1;
					end
				8:	write_fls <= 0;
				9:	write_fls <= 0;
				10:	write_fls <= 0;
				11:	write_fls <= 1;
				12:
					begin
						if (data_cnt == flush_burst_cnt)
							write_fls <= 0;
						else
							write_fls <= 1;	
					end
				13:	write_fls <= 0;
				14:	write_fls <= 0;
				15:	write_fls <= 1;	
				16:
					begin
						if (data_cnt == flush_burst_cnt)
							write_fls <= 0;
						else
							write_fls <= 1;	
					end
				17:	write_fls <= 0;
				default:	write_fls <= 0;
			endcase
		end
	
	wire	flush_done;
	assign	flush_done = (c_state_flush == 8) ?  1 : 0;
	//===============================================================================//
	
	wire	[31:0]last_remain_address;
	assign	last_remain_address = REAL_END_ADDRESS - norm_address;
	
	//	Flush Burst count
	reg	[7:0]flush_burst_cnt;
	always@(posedge iCLK)
		begin
			if (c_state_flush == 10)
				flush_burst_cnt <= rdusedw;
			else if (c_state_flush == 14)
				flush_burst_cnt <= last_remain_address[12:5];
			else if (c_state_flush == 5)
				flush_burst_cnt <= 1;
		end
	
	//	flush added address
	reg	[31:0]flush_added_addr;
	always@(posedge iCLK)
		begin
			if (c_state_flush == 10)
				flush_added_addr <= {rdusedw, 5'h0};
		end
	
		
	//	normal address + remain bytes of FIFO 
	wire	[31:0]rdusedw_address;
	assign	rdusedw_address = { 20'h0 , rdusedw, 5'h0 };
	
	wire	[31:0]nor_rm_addr;	
	assign	nor_rm_addr = norm_address + rdusedw_address;
		
		
		
	
	reg	[3:0]usedw_cnt;
	always@(posedge iCLK)
		begin
			if ( (~iRST_n) || (c_state_flush == 3) || (c_state_flush == 5) )
				usedw_cnt <= 0;
			else if (c_state_flush == 4)
				usedw_cnt <= usedw_cnt + 1;
		end
	
	//////////////////////////////////////////////////////
	/////			Byte enable selector 			//////
	//////////////////////////////////////////////////////
	reg		[31:0]byte_en_value;
	always@(wrusedw)
		begin
			case (wrusedw)
				1:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h1;
						2:	byte_en_value <= 32'h3;
						4:	byte_en_value <= 32'hf;
						8:	byte_en_value <= 32'hff;
						16:	byte_en_value <= 32'hffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				2:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h3;
						2:	byte_en_value <= 32'hf;
						4:	byte_en_value <= 32'hff;
						8:	byte_en_value <= 32'hffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				3:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h7;
						2:	byte_en_value <= 32'h3f;
						4:	byte_en_value <= 32'hfff;
						8:	byte_en_value <= 32'hffffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				4:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'hf;
						2:	byte_en_value <= 32'hff;
						4:	byte_en_value <= 32'hffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				5:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h1f;
						2:	byte_en_value <= 32'h3ff;
						4:	byte_en_value <= 32'hfffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				6:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h3f;
						2:	byte_en_value <= 32'hfff;
						4:	byte_en_value <= 32'hffffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				7:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h7f;
						2:	byte_en_value <= 32'h3fff;
						4:	byte_en_value <= 32'hfffffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				8:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'hff;
						2:	byte_en_value <= 32'hffff;
						4:	byte_en_value <= 32'hffff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase	
				9:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h1ff;
						2:	byte_en_value <= 32'h3ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase		
				10:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h3ff;
						2:	byte_en_value <= 32'hfffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase		
				11:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h7ff;
						2:	byte_en_value <= 32'h3fffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase		
				12:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'hfff;
						2:	byte_en_value <= 32'hff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase	
				13:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h1fff;
						2:	byte_en_value <= 32'h3ff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase		
				14:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h3fff;
						2:	byte_en_value <= 32'hfff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase		
				15:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h7fff;
						2:	byte_en_value <= 32'h3fff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase		
				16:
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'hffff;
						2:	byte_en_value <= 32'hffff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase		
				17:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h1ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				18:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h3ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				19:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h7ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				20:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'hfffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				21:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h1f_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				22:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h3f_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				23:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h7f_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				24:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'hff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				25:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h1ff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				26:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h3ff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				27:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h7ff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				28:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'hfff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				29:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h1fff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				30:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h3fff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase
				31:	
					case (BYTE_EN_PARA)
						1:	byte_en_value <= 32'h7fff_ffff;
						default:	byte_en_value <= 32'hffff_ffff;
					endcase																
				default:	byte_en_value <= 32'hffff_ffff;
			endcase
		end
	
	
	reg	[31:0]byte_enable;
	always@(c_state_flush)
		begin
			if ( (c_state == 10) || (c_state == 11) )
				byte_enable <= last_word_bytenable;
			else if ( (c_state_flush == 6) || (c_state_flush == 7) )
				byte_enable <= byte_en_value;
			else if ( (rm_byen) & (write) )
				byte_enable <= ~rm_byte_buf;
			else
				byte_enable <= 32'hffff_ffff;
		end
		
	reg	rm_byen;
	always@(posedge iCLK)
		begin
			if (~iRST_n)
				rm_byen <= 0;
			else if (c_state_flush == 6)
				rm_byen <= 1;
			else if (c_state == 2)
				rm_byen <= 0;	
		end
	
	reg	[31:0]rm_byte_buf;
	always@(posedge iCLK)
		begin
			if (~iRST_n)
				rm_byte_buf <= 0;
			else if (c_state_flush == 6)
				rm_byte_buf <= byte_enable;
		end
	
	
	
	
	//	flop out
//	reg	oCS_fls;
//	reg	oWRITE_fls;
//	always@(posedge iCLK)
//		begin
//			oCS_fls <= write_fls;
//			oWRITE_fls <= write_fls;
//			oBYTE_ENABLE <= byte_enable;
//		end
	
	wire	oCS_fls;
	wire	oWRITE_fls;
	assign	oCS_fls = write_fls;
	assign	oWRITE_fls = write_fls;
	assign	oBYTE_ENABLE = byte_enable;
	

		
	////////////////////////////////////////////
	////////////////////////////////////////////

	reg		[rddw_num:0]data_cnt;
	reg		[3:0]c_state;
	reg		[3:0]n_state;
	
	always@(posedge iCLK)
		begin
			if ( (~iRST_n) || (~iRST_n_F) )
				c_state <= 0;
			else
				c_state <= n_state;
		end
	
	wire	[31:0]next_address;
	assign	next_address = norm_address + INC_ADDR;
	
	
	//	use to remain bytes at last write 
	wire	[31:0]end_rm_bytes;
	assign	end_rm_bytes = REAL_END_ADDRESS - norm_address;
	
	wire	[12:0]add_rm_address;
	assign	add_rm_address = {end_rm_bytes[12:5], 5'h0};
	
	wire	[31:0]end_rm_address;
	assign	end_rm_address = REAL_END_ADDRESS - add_rm_address;
	
	reg		[4:0]lsb_rm_bytes;
	always@(posedge iCLK)
		begin
			if (c_state == 6)
				lsb_rm_bytes <= end_rm_bytes[4:0];
		end
	
	//	burst counter <= end_rm_bytes[12:5]
	reg		rm_burst_flag;
	always@(posedge iCLK)
		begin
			if ( (~iRST_n) || (~iRST_n_F) )
				rm_burst_flag <= 0;
			else if (c_state == 6)
				rm_burst_flag <= 1;
			else if (c_state == 8)
				rm_burst_flag <= 0;
		end
	
	//	write last one word
	reg		last_word;
	always@(posedge iCLK)
		begin
			if ( (~iRST_n) || (~iRST_n_F) )
				last_word <= 0;
			else if (c_state == 9)
				last_word <= 1;
			else if (c_state == 11)
				last_word <= 0;
		end
	
	
	always@(c_state or rdusedw or data_cnt or iWAIT_REQ)
		begin
			case (c_state)
				0:	n_state <= 1;
				1:
					if ( (rdusedw >= half_of_depth) & (c_state_flush == 1) ) 
						begin
							if (next_address > REAL_END_ADDRESS )
								n_state <= 6;
							else 
								n_state <= 2;
						end
					else
						n_state <= 1;
				2:	n_state <= 3;
				3:	
//					if (~iWAIT_REQ)
						if (data_cnt == BURST_CNT)
							n_state <= 4;
						else
							n_state <= 3;		
//					else	
//						n_state <= 3; 
				4:	n_state <= 1; 
				5:	n_state <= 2;
				6:	n_state <= 7;	// last write need to process
				7:
//					if (~iWAIT_REQ)
						if (data_cnt == end_rm_bytes[12:5])
							n_state <= 8;
						else
							n_state <= 7;		
//					else	
//						n_state <= 7;
				8:	
					if (lsb_rm_bytes != 0)	
						n_state <= 9;
					else	
						n_state <= 1; 
				9:		n_state <= 10;		// write one data with byte enable
				10:		n_state <= 11;	
				11:	
					if (~iWAIT_REQ)
						n_state <= 12;
					else
						n_state <= 11; 
				12:	n_state <= 1;
				default:n_state <= 0;
			endcase
		end
	
	//	Cross Over address burst flag
	reg	crs_addr_flag;
	always@(posedge iCLK)
		begin
			if (~iRST_n)
				crs_addr_flag <= 0;
			else if (c_state == 4)
				crs_addr_flag <= 0;
			else if (c_state == 5)
				crs_addr_flag <= 1; 
		end
	
	
	//	flop out
//	reg	CS_normal;
//	reg	WRITE_normal;
//	always@(posedge iCLK)
//		begin
//			CS_normal <= write;
//			WRITE_normal <= write;
//		end

	wire	CS_normal;
	wire	WRITE_normal;
	assign	CS_normal = write;
	assign	WRITE_normal = write;

	reg	write;
	always@(c_state)
		begin
			case (c_state)
				0:	write <= 0;
				1:	write <= 0;
				2:	write <= 1;
				3:	
//					if (~iWAIT_REQ)
						if (data_cnt == BURST_CNT)
							write <= 0;
						else
							write <= 1;	
//					else	
//						write <= 1;	
				4:	write <= 0;
				5:	write <= 0;
				6:	write <= 1;
				7:	
//					if (~iWAIT_REQ)
						if (data_cnt == end_rm_bytes[12:5])
							write <= 0;
						else
							write <= 1;	
//					else	
//						write <= 1;
				8:	write <= 0;
				9:	write <= 0;
				10:	write <= 1;
				11:
					if (~iWAIT_REQ)
						write <= 0;
					else
						write <= 1;
				12:	write <= 0;
				default:write <= 0;
			endcase
		end
	
	
	//	Address operation 
	reg		[31:0]flush_address;
	reg		[31:0]norm_address;
	always@(posedge iCLK)
		begin
			if ( (~iRST_n) || (~iRST_n_F) || (c_state == 0) )
				norm_address <= STARTING_ADDRESS;
//			else if (flush_done)
//				norm_address <= flush_address;
			else if (c_state == 4)					//( (oWRITE) & (~iWAIT_REQ) )
				begin
					if (crs_addr_flag)
						norm_address <= flush_address;
					else if (norm_address == END_ADDR)
						norm_address <= STARTING_ADDRESS;
					else
						norm_address <= norm_address + INC_ADDR;
				end
			 
			else if (c_state == 8)
				begin
					if (norm_address == end_rm_address)
						norm_address <= STARTING_ADDRESS;
					else
						norm_address <= norm_address + add_rm_address;
				end
			else if ( (c_state == 12) || (c_state_flush == 17) )
				norm_address <= STARTING_ADDRESS;
			else if (c_state_flush == 13)
				norm_address <= norm_address + flush_added_addr;
		end
		
		
		
//	always@(posedge iCLK)
//		begin
//			if ( (~iRST_n) || (~iRST_n_F) || (c_state == 0) )
//				flush_address <= STARTING_ADDRESS;
//			else if ( (oWRITE) & (~iWAIT_REQ) & (c_state_flush !== 7) )
//			 begin
//				if (flush_address == FLUSH_END_ADDRESS)
//					flush_address <= STARTING_ADDRESS;
//				else
//					flush_address <= flush_address + MEM_LOCAL_WIDTH_BYTES;
//			 end
//		end
	
	
	
	always@(posedge iCLK)
		begin
			if ( (~iRST_n) || (~iRST_n_F) )
				data_cnt <= 0;
			else if ( ( (c_state == 1) & (c_state_flush == 1) ) || (c_state_flush == 17) || (c_state_flush == 13))
//				if (rdusedw >= half_of_depth)
//					data_cnt <= 1;
//				else
					data_cnt <= 0;
//			else if ( (c_state == 2) || (c_state == 3) || (c_state == 6) || (c_state == 7) || (c_state_flush == 12))
			else if (oWRITE)
				if (~iWAIT_REQ)
					data_cnt <= data_cnt + 1;
		end
	
	
	
	
	
	
	
//	//test
//	output reg	[23:0]test_fifo_read;
//	always@(posedge iCLK)
//		begin
//			if ( (~iRST_n) || (~iRST_n_F) )
//				test_fifo_read <= 0;
//			else if (fifo_read)
//				test_fifo_read <= test_fifo_read + 1;
//		end
//	
//	//test
	
	reg	[31:0]last_word_bytenable;
	always@(end_rm_bytes[4:0])
		begin
			case (end_rm_bytes[4:0])
				0:	last_word_bytenable <= 0;
				1:	last_word_bytenable <= 32'h1;
				2:	last_word_bytenable <= 32'h3;	
				3:	last_word_bytenable <= 32'h7;
				4:	last_word_bytenable <= 32'hf;
				5:	last_word_bytenable <= 32'h1f;
				6:	last_word_bytenable <= 32'h3f;
				7:	last_word_bytenable <= 32'h7f;
				8:	last_word_bytenable <= 32'hff;
				9:	last_word_bytenable <= 32'h1ff;
				10:	last_word_bytenable <= 32'h3ff;
				11:	last_word_bytenable <= 32'h7ff;
				12:	last_word_bytenable <= 32'hfff;
				13:	last_word_bytenable <= 32'h1fff;
				14:	last_word_bytenable <= 32'h3fff;
				15:	last_word_bytenable <= 32'h7fff;
				16:	last_word_bytenable <= 32'hffff;
				17:	last_word_bytenable <= 32'h1ffff;
				18:	last_word_bytenable <= 32'h3ffff;
				19:	last_word_bytenable <= 32'h7ffff;
				20:	last_word_bytenable <= 32'hfffff;
				21:	last_word_bytenable <= 32'h1fffff;
				22:	last_word_bytenable <= 32'h3fffff;
				23:	last_word_bytenable <= 32'h7fffff;
				24:	last_word_bytenable <= 32'hffffff;
				25:	last_word_bytenable <= 32'h1ffffff;
				26:	last_word_bytenable <= 32'h3ffffff;
				27:	last_word_bytenable <= 32'h7ffffff;
				28:	last_word_bytenable <= 32'hfffffff;
				29:	last_word_bytenable <= 32'h1fffffff;
				30:	last_word_bytenable <= 32'h3fffffff;
				31:	last_word_bytenable <= 32'h7fffffff;
				default: last_word_bytenable <= 32'hffffffff;
			endcase
		end
	
	
	
	
	
	
	
endmodule 