#ifndef ADXL345_H_
#define ADXL345_H_

#include "terasic_includes.h"

#define ON  0x01
#define OFF 0x00

//define all the ADXL345 inner registers
#define DEVID           0x00    //Device ID
#define THRESH_TAP      0x1D    //Tap threshold
#define OFSX            0x1E    //X-axis offset
#define OFSY            0x1F    //Y-axis offset
#define OFSZ            0x20    //Z-axis offset
#define DUR             0x21    //Tap duration
#define LATENT          0x22    //Tap latency
#define WINDOW          0x23    //Tap window
#define THRESH_ACT      0x24    //Activity threshold
#define THRESH_INACT    0x25    //Inactivity threshold
#define TIME_INACT      0x26    //Inactivity time
#define ACT_INACT_CTL   0x27    //Axis enable control for activity and inactivity detection
#define THRESH_FF       0x28    //Free-fall threshold
#define TIME_FF         0x29    //Free-fall time
#define TAP_AXES        0x2A    //Axis control for single tap/double tap
#define ACT_TAP_STATUS  0x2B    //Source of single tap/double tap
#define BW_RATE         0x2C    //Data rate and power mode control
#define POWER_CTL       0x2D    //Power-saving features control
#define INT_ENABLE      0x2E    //nterrupt enable control
#define INT_MAP         0x2F    //Interrupt mapping control
#define INT_SOURCE      0x30    //Source of interrupts
#define DATA_FORMAT     0x31    //Data format control
#define DATAX0          0x32    //X-Axis Data 0
#define DATAX1          0x33    //X-Axis Data 1
#define DATAY0          0x34    //Y-Axis Data 0
#define DATAY1          0x35    //Y-Axis Data 1
#define DATAZ0          0x36    //Z-Axis Data 0
#define DATAZ1          0x37    //Z-Axis Data 1
#define FIFO_CTL        0x38    //FIFO control
#define FIFO_STATUS     0x39    //FIFO status

/*
 * for interrupt vector the bit define in register INT_SOURCE
 * more specified information refer to the ADXL345 datasheet
 */
 #define DATA_READY  7
 #define SINGLE_TAP  6
 #define DOUBLE_TAP  5
 #define ACTIVITY    4
 #define INACTIVITY  3
 #define FREE_FALL   2
 #define EATERMARK   1
 #define OVERRUN     0
 
 

//define the I2C  slave address and device address
#define ADXL345_ID  0xE5
#define ADXL345_WRITE_ADDR    0xA6
#define ADXL345_READ_ADDR    0xA7

//define ADXL345's I2C sianal wire address according to the system.ptf file 

#define ADXL345_SCL_BASE G_SENSOR_I2C_SCL_BASE
#define ADXL345_SDA_BASE G_SENSOR_I2C_SDA_BASE


//the control Functions
bool ADXL345_init(void);
void ADXL345_test(void);
bool Gsensor_selftest(alt_u8 select);
bool Gsensor_init(void);
bool Gsensor_uinit(void);
bool Gsensor_getXYZ(alt_16 *x,alt_16 *y,alt_16 *z);
bool Gsensor_SETXYZ_BIAS(alt_8 x_bias,alt_8 y_bias,alt_8 z_bias);
bool Gsensor_AutoCalibration(void);
bool Gsensor_interrupt_ISR(void);

bool measure_mode(void);
bool standby_mode(void);
bool set_range(alt_u8 range);
bool get_range(alt_u8 *range);

void I2C_write_Reg(alt_u8 address,alt_u8 value);
bool I2C_read_Reg(alt_u8 address,alt_u8 *value);
alt_u8 I2C_write_nReg(alt_u8 readaddr,alt_u8 readdata[],alt_u8 len);
alt_u8 I2C_read_nReg(alt_u8 readaddr,alt_u8 readdata[],alt_u8 len);

alt_u8 read_all_ADXL3435_register(void);

/*
 * for debug
 */
 void RegPrintf(alt_u8 RegAddr);
 


#endif /*ADXL345_H_*/
